<?php
/**
 * Plugin Name:     Scraping, AI e Integración by Sabandijers
 * Plugin URI:      https://sabandijers.club/directo/scraping-y-apis-code-vs-no-code/
 * Description:     Plugin sabandijer hecho en el directo del 19 de abril. Más info en https://sabandijers.club/directo/scraping-y-apis-code-vs-no-code/
 * Author:          Álvaro de Lera
 * Author URI:      https://sabandijers.club/directo/scraping-y-apis-code-vs-no-code/
 * Text Domain:     sabandijas
 * Domain Path:     /languages
 * Version:         0.1.0
 *
 * @package         Sabandijas
 */



 /**
 * Integramos librería OpenAI y llamadas a su API
 * Recibe el prompt y el contenido y devuelve el texto generado por ChatGPT.
 */

require __DIR__ . '/vendor/autoload.php'; // remove this line if you use a PHP Framework.
use Orhanerday\OpenAi\OpenAi;

function sabandijas_ai($content){
    if( !defined('OPENAI_API_KEY') ){
        return $content;
    }
    $open_ai = new OpenAi(OPENAI_API_KEY);

    //Si no hay un prompt guardado en la base de datos, usamos este por defecto
    $prompt = get_option('sabandijas_prompt');
    if( empty($prompt) ){
        $prompt = 'Traduce este texto de ingles a español y dame el resultado maquetado en código HTML, sin añadir más comentarios que no sea el código. El texto es el siguiente: \n\n ';
    }

    try {
        // Aquí usamos el modelo Davinci, pero puedes usar cualquiera de los modelos de OpenAI
        $complete = $open_ai->completion([
            'model' => 'text-davinci-003',
            'prompt' => $prompt. '\n\n ' .$content,
            'temperature' => 0.9,
            'max_tokens' => 1000,
            'frequency_penalty' => 0,
            'presence_penalty' => 0.6,
        ]);

    } catch (\Throwable $th) {
       return 'Ha ocurrido un error con la API de OpenAI: ' . $th->getMessage();
    }

    $d = json_decode($complete);

    // Descomenta la siguiente línea para ver el resultado de la API en el log de errores
    // error_log(print_r($d, true));

    // Get Content
    return $d->choices[0]->text;
}

//Ahora en cualquier sitio que quieras utilizar la API con tu prompt, simplemente llama a esta función y pásale un contenido:
//sabandijas_ai($content);






/**
 * Integramos la librería Simple HTML DOM y ajustamos las opciones de Scraping
 * - Según la web a scrapear hay que ajustar las opciones que necesitamos.
 * - La información se puede guardar en JSON, CSV o Google Sheets.
 * - Si el algún momento la web cambia haremos que se nos informe por email.
 * Documentación: https://simplehtmldom.sourceforge.io/docs/1.9/index.html
 */
include_once('vendor/simplehtmldom/simple_html_dom.php');

// ES MEJOR USAR LA FUNCIÓN CURL DE PHP PARA COGER EL CONTENIDO DE LA WEB EN VEZ DE "file_get_html()" DE SIMPLE HTML DOM, FUNCIONA MEJOR Y FALLA MENOS
function get_web_page( $url )
{
        $user_agent='Mozilla/5.0 (Windows NT 6.1; rv:8.0) Gecko/20100101 Firefox/8.0';
        $options = array(
            CURLOPT_CUSTOMREQUEST  => "GET",        //set request type post or get
            CURLOPT_POST           => false,        //set to GET
            CURLOPT_USERAGENT      => $user_agent, //set user agent
            CURLOPT_RETURNTRANSFER => true,     // return web page
            CURLOPT_HEADER         => false,    // don't return headers
            CURLOPT_FOLLOWLOCATION => true,     // follow redirects
            CURLOPT_ENCODING       => "",       // handle all encodings
            CURLOPT_AUTOREFERER    => true,     // set referer on redirect
            CURLOPT_CONNECTTIMEOUT => 120,      // timeout on connect
            CURLOPT_TIMEOUT        => 120,      // timeout on response
            CURLOPT_MAXREDIRS      => 5,       // stop after 5redirects
        );

        $ch      = curl_init( $url );
        curl_setopt_array( $ch, $options );
        $content = curl_exec( $ch );
        $err     = curl_errno( $ch );
        $errmsg  = curl_error( $ch );
        $header  = curl_getinfo( $ch );
        curl_close( $ch );

        $header['errno']   = $err;
        $header['errmsg']  = $errmsg;
        $header['content'] = $content;
        return $content;
}


/**
 * Aquí está la función utilizada en el directo para la web concreta que queríamos scrapear: https://proplugindirectory.com
 * Tendrás que cambiarla a tu gusto para scrapear la web que quieras y guardar los elementos que quieras llevar a tu web.
 * EJEMPLO: TODOS LOS ENCABEZADOS
 * foreach($html->find('h1,h2,h3,h4,h5') as $found) {
 *   echo $found->tag.' - ' . $found->plaintext .'<br>-------<br>';
 * }
 */

function scrapeWebsite(){
    // Cogemos todas las URLs del sitemap y las guardamos en un array
    $sitemap = str_get_html(get_web_page('https://proplugindirectory.com/download-sitemap.xml'));
    $urls = [];
    foreach($sitemap->find('loc') as $url) {
        $urls[] = $url->plaintext;
    }

    // Recorremos el array de URLs y hacemos scraping de cada una de ellas
    // También podemos guardar esas URLs en un CSV o Google Sheets para tener un registro de las URLs que hemos scrapeado y saber en el futuro si hay nuevas URLs que scrapear
    //
    // PROBLEMA: COMO SON MUCHOS PLUGINS, EL SCRIPT TARDARÍA DEMASIADO TIEMPO Y EL SERVIDOR LO CERRARÍA POR LÍMITE DE TIEMPO O LA WEB NOS BLOQUEARIA POR HACER DEMASIADAS PETICIONES
    // SOLUCIONES:
    //  - HACERLO POR BLOQUES DE PLUGINS. DE 30 EN 30, 50 EN 50... HACES PRUEBAS PARA VER CUÁNTO SE TARDARÍA
    //  - PONER UN TIEMPO DE ESPERA ALEATORIO ENTRE CADA PETICION DE ALGUNOS SEGUNDOS (FUNCIÓN "SLEEP" DE PHP)
    //  - UTILIZAR UN CRON QUE VAYA HACIENDO LLAMAS CADA CIERTO TIEMPO
    //  - PARA EL BLOQUEO DE LA WEB: UTILIZAR PROXIES

    $pluginsWithInfo = [];
    $i = 0;
    foreach($urls as $url) {
        //Aquí estamos cogiendo solo 5 plugins para hacer pruebas y no tardar tanto tiempo
        if($i < 5) {
            $html = str_get_html(get_web_page($url));
            // Hacemos scraping de la URL concreta y guardamos los datos en un array
            $pluginsWithInfo[] = scrapePluginURL($html);
            $i++;
        }

    }

    // GUARDAMOS LOS DATOS EN JSON  ( O EXCEL / CSV / MYSQL... DONDE QUERAMOS) 
    // Aquí lo guardamos como JSON
    $jsonString = json_encode($pluginsWithInfo, JSON_PRETTY_PRINT);
    file_put_contents($_SERVER['DOCUMENT_ROOT'].'/wp-content/plugins/sabandijas/json/plugins.json', $jsonString);

    // Aquí lo guardamos como CSV, para poder utilizarlo con WP ALL IMPORT
    $csvFile = fopen($_SERVER['DOCUMENT_ROOT'].'/wp-content/plugins/sabandijas/json/plugins.csv', 'w');
    // Escribir la codificación UTF-8 BOM (Byte Order Mark) para asegurar la correcta interpretación de los caracteres especiales
    fprintf($csvFile, chr(0xEF).chr(0xBB).chr(0xBF));
    // Escribir los encabezados del CSV
    $headers = array('ID', 'Titulo', 'Extracto', 'Descripción', 'Vendor', 'Imagen', 'Precio');
    fputcsv($csvFile, $headers);
    // Iterar a través de los datos y escribir las filas en el archivo CSV
    foreach ($pluginsWithInfo as $id => $plugin) {
        fputcsv($csvFile, array($id, $plugin['title'], $plugin['short_description'], $plugin['description'], $plugin['vendor'], $plugin['img'], $plugin['price']));
    }
    // Cerrar el archivo CSV
    fclose($csvFile);



}

// Hacemos scraping de la URL
function scrapePluginURL($html) {
    $features = [];
    foreach ($html->find('.download-features li') as $feature) {
        $features[] = $feature->plaintext;
    }

    //Devolvemos todos los elementos que queremos scrapear e importar a nuestra web
    $plugin = [
        'title' => trim($html->find('h1.entry-title',0)->plaintext),
        'short_description' => trim($html->find('.entry-subtitle',0)->plaintext),
        'vendor' => trim($html->find('#ppd-plugin-details-shop',0)->plaintext),
        'img' => $html->find('img.wp-post-image',0)->src,
        'description' => trim($html->find('div.post-text',0)->plaintext),
        //'features' => $features,
        'price' => rand (0,100) // SUPONGAMOS QUE TIENE EL PRECIO PUESTO
    ];

    return $plugin;
}

// Quitar comentario para ponerse a scrapear
//scrapeWebsite();



/**
 * EJEMPLOS PARA SCRAPEAR OTRO TIPO DE PÁGINAS, CUANDO NO TENEMOS SITEMAP
 */

// Página de categorías

/*
$html = str_get_html(file_get_contents('html/home.html'));

$categories = [];

// Selecciono el ID del listado de categorías y hago bucle por todos sus elementos <li> que contienen un <a> y un listado <ul> con las subcategorías

foreach($html->find('#ppd-home-categories-list li') as $found) {
// Solo las categorías padre 
$link = $found->find('> a', 0);
$sub = [];
//Las categorías hija de cada padre
foreach($found->find('li') as $li) {
    $sublink = $li->find('a', 0);
    $sub[] = [
        'href' => $sublink->href,
        'title' => $sublink->plaintext,
        'description' => $sublink->title,
    ];
}

//Montamos un array con todo
$categories[] = [
    'href' => $link->href,
    'title' => $link->plaintext,
    'description' => $link->title,
    'sub' => $sub
];
}

echo '<pre>';
print_r($categories);

// GUARDAMOS LOS DATOS EN JSON  ( O EXCEL / CSV / MYSQL... DONDE QUERAMOS) 

$jsonString = json_encode($categories, JSON_PRETTY_PRINT);
$fp = fopen('json/categorias.json', 'w');
fwrite($fp, $jsonString);
fclose($fp);

*/

/*---------------------------------------------------------------------------------------------------*/



// Página archive con todos los plugins
/*

// $url = 'https://proplugindirectory.com/plugins/';
// $html = str_get_html(get_web_page($url));

// HACER PARA CADA PAGINACIÓN https://proplugindirectory.com/plugins/page/2/ --- https://proplugindirectory.com/plugins/page/25/


$html = str_get_html(file_get_contents('html/allplugins.html'));

$plugins = [];

// BUSCO EL ELEMENTO QUE SEA PADRE DE TODO, LA CLASE portfolio-wrapper, CADA PLUGIN ES UN <ARTICLE>

foreach($html->find('.portfolio-wrapper article') as $found) {
    $link = $found->find('a', 0);

    $description = $found->find('.entry-excerpt',0);
    // LA DESCRIPCIÓN INCLUYE LOS ENLACES A RRSS, LOS SELECCIONO Y LOS ELIMINO DEL TEXTO DE LA DESCRIPCIÓN
    $rrss = $description->find('.sd-content', 0)->plaintext;
    $description = trim(str_replace($rrss, '', $description->plaintext));

    $plugins[] = [
        'img' => $link->find('img', 0)->src,
        'title' => trim($found->find('h3', 0)->plaintext),
        'description' => $description,
        'category' => trim($html->find('.post-meta',0)->plaintext),
        'vendor' => trim($found->find('.download-meta-name',0)->plaintext),
    ];
}

// GUARDAMOS LOS DATOS EN JSON  ( O EXCEL / CSV / MYSQL... DONDE QUERAMOS)

$jsonString = json_encode($plugins, JSON_PRETTY_PRINT);
$fp = fopen('json/todosplugins.json', 'w');
fwrite($fp, $jsonString);
fclose($fp);

*/



// ----------------------------------------------------------------------------------------------------------------------------
// El código a partir de aquí es para la instalación y configuración del plugin, no es necesario que cambies nada de aquí para abajo
// ----------------------------------------------------------------------------------------------------------------------------





/**
 * Este código es para redirigir a la página de configuración del plugin tras la activación
 */
register_activation_hook(__FILE__, 'redirect_after_activation');

function redirect_after_activation() {
    add_option('redirect_after_activation_option', true);
}

add_action('admin_init', 'activation_redirect');

function activation_redirect() {
    if (get_option('redirect_after_activation_option', false)) {
        delete_option('redirect_after_activation_option');
        exit(wp_redirect(admin_url( 'options-general.php?page=sabandi-options' )));
    }
}



/**
 * Añadimos página de opciones en el menú de Ajustes de WordPress
 * - Opción para añadir el prompt a enviar a OpenAI
 * - Información sobre cómo añadir la API key de OpenAI
 */

 if(is_admin()){
    add_action('admin_menu', 'sabandi_options_page');
    add_action('admin_init', 'sabandi_options_page_init');
}

// Crear página de opciones en el menú de administración
function sabandi_options_page() {
    add_options_page(
        'Ajustes',
        'Scraping, AI e Integración by Sabandijers',
        'manage_options',
        'sabandi-options',
        'sabandi_options_page_html'
    );
}


// HTML para la página de opciones
function sabandi_options_page_html() {
    if (!current_user_can('manage_options')) {
        return;
    }
    ?>
    <div class="wrap">
        <h1>Scraping, AI e Integración by Sabandijers</h1>
        <h3>Instrucciones de uso</h3>
        <p>Este plugin tiene 2 partes que pueden funcionar juntas o por separado, la de scrapping y la de conexión con la API de ChatGPT. Enseñamos a utilizarlo en el directo <a href="https://sabandijers.club/directo/scraping-y-apis-code-vs-no-code/" target="_blank">de Scrapping y APIs</a>.</p>
        <p><strong>No está creado para usarlo tal cual</strong>, ya que de momento solo funciona con la página de pruebas que usamos en el directo.</p>
        <p><strong>Lo mejor es que veas el directo</strong> pero igualmente te dejamos aquí un resumen de uso:</p>
        <ul style="list-style-type:disc;margin-left:20px;">
            <li>Este plugin está pensado para que te sirva de base y lo termines de customizar a tu gusto, por lo que <strong>es necesario que sepas algo de HTML yprogramación en PHP</strong>.</li>
            <li><strong>Si no sabes programar, entonces mejor usa Make</strong> o cualquier otra app de conexión de APIs, tal como explica en el directo Ángel Cano.</li>
            <li>Para la parte de scrapping usamos la librería <a href="https://simplehtmldom.sourceforge.io/" target="_blank">Simple HTML DOM de PHP</a>.</li>
            <li>Siempre puedes scrappear con otra app aparte, guardar el resultado en un archivo CSV en /wp-content/plugins/sabandijas/json/nombre-archivo.csv, y hacer referencia a él en el código del plugin.</li>
            <li>Para utilizar la parte de ChatGPT, <strong>guarda primero el prompt que quieras utilizar aquí abajo</strong> (o usa el que hay por defecto).</li>
            <li>El código del plugin está todo comentado para explicarte la función de cada pieza de código y que sepas modificarlo a tu gusto.</li>
            <li>Disfrútalo y haz el bien. O no.</li>
        </ul>
        <form action="options.php" method="post">
            <?php
            settings_fields('sabandi_options');
            echo '<p>Este plugin utiliza la API de OpenAI para generar contenido a partir de un texto de entrada. <strong>Para que funcione debes añadir primero a tu archivo wp-config-php tu API key de OpenAI</strong>, tal que así:</p>';
            echo '<pre style="border:1px dashed darkgrey;background-color:#eee;display:inline-block;padding:0 10px;">define(\'OPENAI_API_KEY\', \'sk-xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\');</pre>';
            do_settings_sections('sabandi-options');
            submit_button('Guardar ajustes');
            ?>
        </form>
    </div>
    <?php
}

// Registrar configuraciones y campos
function sabandi_options_page_init() {
    register_setting('sabandi_options', 'sabandi_options_prompt');

    add_settings_section(
        'sabandi_options_section',
        'Ajustes',
        '',
        'sabandi-options'
    );

    add_settings_field(
        'sabandi_options_prompt',
        'Prompt para enviar a ChatGPT',
        'sabandi_options_prompt_callback',
        'sabandi-options',
        'sabandi_options_section'
    );
}

// Mostrar el campo de texto "Prompt"
function sabandi_options_prompt_callback() {
    $prompt_value = get_option('sabandi_options_prompt');
    echo '<textarea name="sabandi_options_prompt" cols="100" rows="5">' . $prompt_value . '</textarea>';
    echo '<p class="description">Prompt que se enviará a la API de OpenAI antes del contenido. Por ejemplo:</p>';
    echo '<p class="description"><i>Traduce este texto de ingles a español y dame el resultado maquetado en código HTML, sin añadir más comentarios que no sea el código. El texto es el siguiente:</i></p>';
}